@@ -90,18 +90,20 @@ printRule(TranslationTableRule *rule, widechar *rule_string) {
 		return 0;
 	default: {
 		int l = 0;
+		if (rule->nocross)
+			for (char *c = "nocross "; *c; c++) rule_string[l++] = *c;
 		const char *opcode = _lou_findOpcodeName(rule->opcode);
 		for (size_t k = 0; k < strlen(opcode); k++) rule_string[l++] = opcode[k];
-		rule_string[l++] = ' ';
+		rule_string[l++] = '\t';
 		for (int k = 0; k < rule->charslen; k++) rule_string[l++] = rule->charsdots[k];
-		rule_string[l++] = ' ';
+		rule_string[l++] = '\t';
 		for (int k = 0; k < rule->dotslen; k++) {
-			rule_string[l] = _lou_getCharFromDots(
+			rule_string[l] = _lou_getCharForDots(
 					rule->charsdots[rule->charslen + k], displayTable);
 			if (rule_string[l] == '\0') {
 				// if a dot pattern can not be displayed, print an error message
 				char *message = (char *)malloc(50 * sizeof(char));
-				sprintf(message, "ERROR: provide a display rule for %s",
+				sprintf(message, "ERROR: provide a display rule for dots %s",
 						_lou_showDots(&rule->charsdots[rule->charslen + k], 1));
 				l = 0;
 				while (message[l]) {
@@ -226,8 +228,6 @@ find_matching_rules(widechar *text, int text_len, widechar *braille, int braille
 			case CTO_EndWord:
 				if (data[-1] != '^' && rule->charslen == text_len) break;
 				goto next_rule;
-			case CTO_NoCross:
-				break;
 			case CTO_Letter:
 			case CTO_UpperCase:
 			case CTO_LowerCase:
@@ -247,13 +247,13 @@ find_matching_rules(widechar *text, int text_len, widechar *braille, int braille
 					(rule->dotslen == braille_len && rule->charslen < text_len))
 				goto inhibit;
 			for (k = 0; k < rule->dotslen; k++)
-				if (_lou_getCharFromDots(rule->charsdots[rule->charslen + k],
+				if (_lou_getCharForDots(rule->charsdots[rule->charslen + k],
 							displayTable) != braille[k])
 					goto inhibit;
 
 			/* don't let this rule be inhibited by an earlier rule */
 			int inhibit_all = 0;
-			if (rule->opcode == CTO_NoCross)
+			if (rule->nocross)
 				for (k = 0; k < rule->charslen - 1; k++)
 					if (data[k + 1] == '>') {
 						if (data[-1] == 'x')
@@ -263,10 +263,9 @@ find_matching_rules(widechar *text, int text_len, widechar *braille, int braille
 					}
 
 			/* fill data */
-			switch (rule->opcode) {
-			case CTO_NoCross:  // deferred: see success
-				break;
-			default:
+			if (rule->nocross)
+				;  // deferred: see success
+			else {
 				k = 0;
 				while (k < rule->charslen - 1) {
 					if (data[k + 1] == '>') {
@@ -291,8 +290,7 @@ find_matching_rules(widechar *text, int text_len, widechar *braille, int braille
 
 		inhibit:
 			debug("** rule inhibited **");
-			switch (rule->opcode) {
-			case CTO_NoCross:
+			if (rule->nocross) {
 				if (rule->charslen < 2) goto abort;
 				/* inhibited by earlier rule */
 				for (k = 0; k < rule->charslen - 1; k++)
@@ -300,19 +298,16 @@ find_matching_rules(widechar *text, int text_len, widechar *braille, int braille
 				data[rule->charslen - 1] = ')';
 				debug("%s", data);
 				goto next_rule;
-			default:
+			} else {
 				goto abort;
 			}
 
 		success:
 			/* fill data (deferred) */
 			if (inhibit_all) data[-1] = '1';
-			switch (rule->opcode) {
-			case CTO_NoCross:
+			if (rule->nocross) {
 				memset(data, '0', rule->charslen - 1);
 				debug("%s", data);
-			default:
-				break;
 			}
 			free(data_save);
 			return 1;
@@ -433,7 +428,6 @@ findRelevantRules(widechar *text, widechar **rules_str) {
 				case CTO_MidWord:
 				case CTO_MidEndWord:
 				case CTO_EndWord:
-				case CTO_NoCross:
 					break;
 				default:
 					goto next_rule;
